V1 = '1'
V2 = '2'
V3 = '3'
V4 = '4'
V5 = '5'
V6 = '6'
V7 = '7'

KNOWN_LEGACY_VERSIONS = {V1, V2, V3, V4, V5, V6}
KNOWN_USER_VERSIONS = {V6, V7}

LATEST_VERSION = V7
LAST_NO_USER_CONFIG_VERSION = V6
FIRST_USER_CONFIG_VERSION = V7
